// C Header File
// Created 4/18/01; 5:08:24 PM
/*
	Link Library Version:beta
	by Greg Dietsche
*/

#ifndef __LinkLibh__
#define __LinkLibh__

#ifndef OPTIMIZE_ROM_CALLS
#error You must use OPTIMIZE_ROM_CALLS when using Link Lib!
#endif

//debugging support
#ifdef DEBUG
	#define debug(s) {printf(s);ngetchx();}
#else
	#define NDEBUG
	#define debug(s) {}
#endif


////////////////////////
//Misc. Stuff        //
//////////////////////
#define LIB_TITLE		"Link Lib"
#define LIB_VERSION 1
#define LNK_TIMEOUT 10	//a value of 20 means 1 second

//Thomas's
//can't be used as a lvalue or rvalue in assignments etc...
//should always be called like ByteSwap(x); nothing more, nothing less
//22628
#define ByteSwap(x) asm volatile ("ror.w #8,%0" \
											 : "=d" (x) : "0" (x))


//mine...
//used to convert words to little-endian format...
//can be a lvalue; can be used as a rvalue...
//makes somewhat larger code...
/*22766
#undef ByteSwap
#define ByteSwap(x) \
({unsigned int __x__=(x); \
	asm("ror.w #8,%1":"=d"(__x__):"d"(__x__)); \
	__x__;})
*/

///////////////////////
//Datatypes         //
/////////////////////

typedef union
{
	unsigned char packet[4];
	struct
	{
		//basic packet info...
		unsigned char devtype;		//see enum DeviceID
		unsigned char cmd;			//see enum LinkCommand
		unsigned int	datalen;	//Same length as the var-link menu would report
		
		//extended info...
		void 					*data;	//pointer to the data to be sent
		//unsigned int	checksum;	//the checksum of the data.
	}extended;
}LinkPacket;




//////////////////
//Enumerations //
////////////////

//fills the devtype variable in a packet
enum DeviceID 
{
		 DEV_C82	=0x02,	//computer to TI-XX
		 DEV_C83	=0x03,
		 DEV_C85	=0x05,
		 DEV_C86	=0x06,
		 DEV_C73	=0x07,
		 DEV_C92P	=0x08,
		 DEV_C92	=0x09,
		 DEV_C83p	=0x73,
		 DEV_CC82	=0x12,	//CBL/CBR to TI-XX
		 DEV_CC83	=0x13,
		 DEV_CC85	=0x15,
		 DEV_CC86	=0x16,
		 DEV_CC73	=0x17,
		 DEV_TI83p=0x73,	//TI-83+ (83+)
		 DEV_TI74	=0x74,	//TI-73  (73)
		 DEV_TI82	=0x82,	//TI-82  (73, 82, 83)
		 DEV_TI83	=0x83,	//TI-83  (73, 82, 83)
		 DEV_TI85	=0x85,	//TI-85  (85, 86)
		 DEV_TI86	=0x86,	//TI-86  (86)
		 DEV_TI92p=0x88,	//TI-92p (89, 92, 92+)
		 DEV_TI92	=0x89,	//TI-92  (89, 92, 92+)
		 DEV_TI89	=0x98		//TI-89  (89, 92+)
};

//Command (cmd) byte values for a link packet
enum LinkCommand
{
		 LNK_READY				=0x0000,//response to LNK_CHECKREADY
		 LNK_NOTREADY			=0x0001,//response to LNK_CHECKREADY
		 LNK_VAR					=0x06,	//Variable Header
		 LNK_CTS					=0x09,	//Clear To Send (waiting for data)
		 //13h backup
		 LNK_SEND					=0x0D,	//?
		 LNK_DATA					=0x15,	//Data
		 LNK_REFUSED			=0x36,	//Calc can't accept transmission
		 LNK_ACK					=0x56,	//Acknowledgment (OK)
		 LNK_ERR					=0x5A,	//bad Checksum; send data again
		 LNK_RDY					=0x68,	//is the calc ready to recieve data?
		 LNK_SCR					=0x6D,	//Request Screen Dump
		 LNK_CONTINUE			=0x78,	//?
		 LNK_CMD					=0x87,	//send a direct command (key press)
		 LNK_EOT					=0x92,	//End of Transmission
		 LNK_REQ					=0xA2,	//Silent - Request Variable
		 LNK_RTS					=0xC9		//Silent - Request to Send Variable
};

//reason why LNK_REFUSED was sent	
enum LinkRefuse
{
		REJECT_EXIT		=0x01,	//abort the link
		REJECT_SKIP		=0x02,	//skip the current var
		REJECT_RAM		=0x03		//reciever out of mem so abort the link
};


//number of bytes that represent the video mem on a specific calculator
enum LCDSizes
{
		LCD_SIZE_82		=768,
		LCD_SIZE_83		=768,
		LCD_SIZE_83p	=768,
		LCD_SIZE_85		=1024,
		LCD_SIZE_86		=1024,
		LCD_SIZE_89		=3840,
		LCD_SIZE_92		=3840,
		LCD_SIZE_92p	=3840
		
};


//enum LinkVariableTypes
//these are different on different calc models...
/*
{
	EXPR	=0,
	LIST	=4,
	MAT		=6
	...see the TIGCC doc's for now
}
*/


/*		General Communication Protocol

Device1:	Var header (Name,type,length)
Device2:	LNK_ACK
Device2:	LNK_CTS
Device1:	LNK_DATA ...send data and checksum...
Device2:	LNK_ACK
Device1:	LNK_EOT
Device2:	LNK_ACK

If you need to send more variables, repeat the above steps...
*/


/////////////////////////
//Function Prototypes //
///////////////////////
short 					GetScreenDump(unsigned char DeviceID, unsigned char* scrbuff, int scrsize);
short 					Link_SendKey(unsigned char DeviceID,short key);
unsigned short	SendLinkCommand(unsigned char id, unsigned char Command);
unsigned short	SendPacket(LinkPacket *packet);
void						AllowLink(short Mode);
unsigned short	Checksum(const unsigned char *const data, unsigned int len);
LinkPacket			*RecievePacket(LinkPacket* Packet);
short 					SendPacketEX(LinkPacket *packet);

#endif
